#
#  Copyright 1999-2002 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc setup_imcagui_callbacks { record_list } {

	find_callback_record $record_list "lambda"       "lambda"
	find_callback_record $record_list "d_spacing"    "d_spacing"

	if { $IMCAGui::show_slit_positions } {
		find_callback_record $record_list "hslit_center" "bxh_center"
		find_callback_record $record_list "vslit_center" "bxv_center"
	}

	find_callback_record $record_list "hslit_size"   "bxh_size"
	find_callback_record $record_list "vslit_size"   "bxv_size"
	find_callback_record $record_list "edge_energy"  "edge_energy"

	if { $IMCAGui::have_undulator } {
	    find_callback_record $record_list "id_ev"          "id_ev"
	    find_callback_record $record_list "id_ev_harmonic" "id_ev_harmonic"
	    find_callback_record $record_list "id_ev_params"   "id_ev_params"
	    find_callback_record $record_list "id_ev_enabled"  "id_ev_enabled"
	}

	find_callback_record $record_list "bx_f3"          "bx_f3"
	find_callback_record $record_list "bx_f4"          "bx_f4"
	find_callback_record $record_list "bx_f5"          "bx_f5"
	find_callback_record $record_list "bx_f6"          "bx_f6"

	start_imcagui_callbacks
}

namespace eval IMCAGui {
	variable program_label

	switch -exact -- $beamline_name {
		dnd5id { set program_label "DND-CAT Sector 5-ID" }
		default { set program_label \
			"Unrecognized beamline '$beamline_name'" }
	}

	if { $beamline_name == "dnd5id" } {
		variable minimum_edge_energy  7000
		variable maximum_edge_energy 14500

		variable have_undulator 1

		variable have_filters 1

		variable show_slit_positions 1
	} else {
		variable minimum_edge_energy  5920
		variable maximum_edge_energy 14000
	}

	# The array edge_array stores information about the available
	# element and edge combinations as read from 'elements.dat'
	#

	variable edge_array
}

############################################################################

proc optimize_menu_handler { frame } {

	# If the menu entries have already been created, just return.

	if [ winfo exists $frame.optimize.tag ] {
		return
	}

	label $frame.optimize.tag	;# Do not pack or grid this.

	set menu $frame.optimize.menu

	$menu add command -label "Manual Optimize..." \
		-command [ itcl::code eval exec "optimize &" ]

	$menu add command -label "Auto Optimize Monochromator..." \
		-command [ itcl::code eval exec "optimize_auto_mono &" ]

	$menu add command -label "Auto Optimize All..." \
		-command [ itcl::code eval exec "optimize_auto_all &" ]

}

############################################################################

proc change_filters { record_list grid } {

	set new_thickness [ $grid.filter_desired get ]

	if { $new_thickness == "" } {
		set new_thickness 0
	}

	# Compute the number of foils rouned to the nearest integer.

	set num_foils [ expr ( round( $new_thickness / 254.0 ) ) ]

	log_puts "new_thickness = $new_thickness, num_foils = $num_foils"

	for { set i 3 } { $i <= 6 } { incr i } {

		set record [ $record_list get_record "bx_f$i" ]

		set bitmask [ expr ( 1 << ( 6 - $i ) ) ]

		if { $num_foils & $bitmask } {
			set relay_value 1
		} else {
			set relay_value 0
		}
#		log_puts \
#		"i = $i, bitmask = $bitmask, relay_value = $relay_value"

		$record relay_command $relay_value
	}
}

############################################################################

proc get_filter_thickness {} {

	global callback_values

	set bx_f3 $callback_values(bx_f3)
	set bx_f4 $callback_values(bx_f4)
	set bx_f5 $callback_values(bx_f5)
	set bx_f6 $callback_values(bx_f6)

	set thickness \
		[ expr ( 8 * $bx_f3 + 4 * $bx_f4 + 2 * $bx_f5 + $bx_f6 ) ]

	set thickness [ expr ( $thickness * 254 ) ]

	return $thickness
}

############################################################################

proc create_filter_frame { frame  record_list} {

        set grid $frame.grid

        frame $grid
        pack $grid -side top

        # Create the filter control grid top labels.

	label $grid.frame_label -text "Attenuation:  "
        label $grid.current -text "Current"
        label $grid.desired -text "Desired"

	grid $grid.frame_label  -row 0 -column 0
        grid $grid.current -row 0 -column 1 -columnspan 2
        grid $grid.desired -row 0 -column 3 -columnspan 2

        # Create the filter row.

        label $grid.filter_label -text "Foil Thickness "
        label $grid.filter_current -fg blue -text "---"
        label $grid.filter_current_units -text "um Al  "        

	iwidgets::entryfield $grid.filter_desired -width 6

	$grid.filter_desired clear
	$grid.filter_desired insert 0 0

	$grid.filter_desired configure \
		-command [ itcl::code change_filters $record_list $grid ]

	$grid.filter_desired configure \
		-invalid invalid_filter_value

	$grid.filter_desired configure -validate { validate_filter_value %P }

        button $grid.change -text "Change" -width 6 \
                -command [ itcl::code change_filters $record_list $grid ]

	$grid.change configure -disabledforeground \
			[ $grid.change cget -foreground ]

        label $grid.filter_desired_units -text "um Al  "

        grid $grid.filter_label         -row 1 -column 0
        grid $grid.filter_current       -row 1 -column 1
        grid $grid.filter_current_units -row 1 -column 2 -sticky w
        grid $grid.filter_desired       -row 1 -column 3
        grid $grid.filter_desired_units -row 1 -column 4 -sticky w
        grid $grid.change               -row 1 -column 5

}

proc validate_filter_value { contents } {
	set valid [ iwidgets::Entryfield::integer $contents ]

	if { $valid == 0 } {
		return 0
	}
	if { $contents < 0 } {
		return 0
	}
	if { $contents > 4000 } {
		return 0
	}
	return 1
}

proc invalid_filter_value {} {
	set t "The filter thickness must be a number from 0 to 4000."

	tk_dialog .invalid_filter_dialog \
	"Invalid Input" $t error 0 "OK"
}

############################################################################

proc update_filter_frame { frame } {
       	set grid $frame.grid

       	set window $grid.filter_current
	set button $grid.change

       	set thickness [ get_filter_thickness ]

         $window configure -text $thickness

       	if { $thickness == "Fault" } {
               	$window configure -fg red
       	} elseif { $thickness == "Moving" } {
               	$window configure -fg limegreen
       	} elseif { $thickness == "0" } {
               	$window configure -fg blue
       	} else {
               	$window configure -fg red
       	}
}

############################################################################

proc create_help_menu { menubar } {

	menu $menubar.help -tearoff 0
	$menubar add cascade -label Help -menu $menubar.help

	$menubar.help add command -label "About IMCA GUI" \
			-command menubar_about_command

	$menubar.help add command -label "Help for DND-CAT 5-ID" \
			-command [ itcl::code display_help "dnd5id.html" ]

	proc menubar_about_command {} {
		set t "IMCA GUI was written by William Lavender based on "
		append t "a design by the IMCA-CAT staff."

		tk_dialog .about_dialog "About IMCA GUI" $t info 0 "Done"
	}
}

