#
#  Copyright 1999-2002 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc setup_imcagui_callbacks { record_list } {

	find_callback_record $record_list "lambda"       "lambda"
	find_callback_record $record_list "d_spacing"    "d_spacing"

	if { $IMCAGui::show_slit_positions } {
		find_callback_record $record_list "hslit_center" "gsh_center"
		find_callback_record $record_list "vslit_center" "gsv_center"
	}

	find_callback_record $record_list "hslit_size"   "gsh_size"
	find_callback_record $record_list "vslit_size"   "gsv_size"
	find_callback_record $record_list "edge_energy"  "edge_energy"
	find_callback_record $record_list "z_trans_enabled" "z_trans_enabled"
	find_callback_record $record_list "y_trans_enabled" "y_trans_enabled"
	find_callback_record $record_list "focus_enabled" "focus_enabled"

	if { $IMCAGui::have_undulator } {
	    find_callback_record $record_list "id_ev"          "id_ev"
	    find_callback_record $record_list "id_ev_harmonic" "id_ev_harmonic"
	    find_callback_record $record_list "id_ev_params"   "id_ev_params"
	    find_callback_record $record_list "id_ev_enabled"  "id_ev_enabled"
	    find_callback_record $record_list "fs1"            "fs1"
	    find_callback_record $record_list "fs2"            "fs2"
	}

	start_imcagui_callbacks
}

namespace eval IMCAGui {
	variable program_label

	set program_label "SER GUI"

	switch -exact -- $beamline_name {
		ser22id { set program_label "SER-CAT Sector 22-ID" }
		default { set program_label \
			"Unrecognized beamline '$beamline_name'" }
	}

	if { $beamline_name == "ser22id" } {
		variable minimum_edge_energy  7000
		variable maximum_edge_energy 14500

		variable have_undulator 1

		variable have_filters 1

		variable show_slit_positions 1
	} else {
		variable minimum_edge_energy  5920
		variable maximum_edge_energy 14000

		variable have_undulator 0

		variable have_filters 1

		variable show_slit_positions 1
	}

	# The array edge_array stores information about the available
	# element and edge combinations as read from 'elements.dat'
	#

	variable edge_array
}

############################################################################

proc optimize_menu_handler { frame } {

	# If the menu entries have already been created, just return.

	if [ winfo exists $frame.optimize.tag ] {
		return
	}

	label $frame.optimize.tag	;# Do not pack or grid this.

	set menu $frame.optimize.menu

	$menu add command -label "Manual Optimize..." \
		-command [ itcl::code eval exec "optimize &" ]

#	$menu add command -label "Auto Optimize Monochromator..." \
#		-command [ itcl::code eval exec "optimize_auto_mono &" ]
#
#	$menu add command -label "Auto Optimize All..." \
#		-command [ itcl::code eval exec "optimize_auto_all &" ]
#
}

############################################################################

proc change_filters { record_list grid } {

	set new_thickness [ $grid.filter_desired get ]

	if { $new_thickness == "" } {
		set new_thickness 0
	}

	for { set i 1 } { $i <= 10 } { incr i } {
		set record [ $record_list get_record "f$i" ]

		set bit_value [ expr ( round( fmod( $new_thickness, 2 ) ) ) ]

		$record write $bit_value

		set new_thickness [ expr ( round( $new_thickness / 2 ) ) ]
	}
}

############################################################################

proc get_filter_thickness {} {

	global callback_values

	set fs1 $callback_values(fs1)
	set fs2 $callback_values(fs2)

	set fs1 [ expr ( round( $fs1 / 4 ) ) ]

	set thickness [ expr ( $fs1 + 64 * $fs2 ) ]

	return $thickness
}

############################################################################

proc create_filter_frame { frame  record_list} {

        set grid $frame.grid

        frame $grid
        pack $grid -side top

        # Create the filter control grid top labels.

	label $grid.frame_label -text "Attenuation:  "
        label $grid.current -text "Current"
        label $grid.desired -text "Desired"

	grid $grid.frame_label  -row 0 -column 0
        grid $grid.current -row 0 -column 1 -columnspan 2
        grid $grid.desired -row 0 -column 3 -columnspan 2

        # Create the filter row.

        label $grid.filter_label -text "Foil Thickness "
        label $grid.filter_current -fg blue -text "---"
        label $grid.filter_current_units -text "foils"        

	iwidgets::entryfield $grid.filter_desired -width 6

	$grid.filter_desired clear
	$grid.filter_desired insert 0 0

	$grid.filter_desired configure \
		-command [ itcl::code change_filters $record_list $grid ]

	$grid.filter_desired configure \
		-invalid invalid_filter_value

	$grid.filter_desired configure -validate { validate_filter_value %P }

        button $grid.change -text "Change" -width 6 \
                -command [ itcl::code change_filters $record_list $grid ]

	$grid.change configure -disabledforeground \
			[ $grid.change cget -foreground ]

        label $grid.filter_desired_units -text "foils"

        grid $grid.filter_label         -row 1 -column 0
        grid $grid.filter_current       -row 1 -column 1
        grid $grid.filter_current_units -row 1 -column 2 -sticky w
        grid $grid.filter_desired       -row 1 -column 3
        grid $grid.filter_desired_units -row 1 -column 4 -sticky w
        grid $grid.change               -row 1 -column 5

}

proc validate_filter_value { contents } {
	set valid [ iwidgets::Entryfield::integer $contents ]

	if { $valid == 0 } {
		return 0
	}
	if { $contents < 0 } {
		return 0
	}
	if { $contents >= 1024 } {
		return 0
	}
	return 1
}

proc invalid_filter_value {} {
	set t "The filter thickness must be a number from 0 to 1023."

	tk_dialog .invalid_filter_dialog \
	"Invalid Input" $t error 0 "OK"
}

############################################################################

proc update_filter_frame { frame } {
       	set grid $frame.grid

       	set window $grid.filter_current
	set button $grid.change

       	set thickness [ get_filter_thickness ]

         $window configure -text $thickness

       	if { $thickness == "Fault" } {
               	$window configure -fg red
       	} elseif { $thickness == "Moving" } {
               	$window configure -fg limegreen
       	} elseif { $thickness == "0" } {
               	$window configure -fg blue
       	} else {
               	$window configure -fg red
       	}
}

############################################################################

proc create_help_menu { menubar } {

	menu $menubar.help -tearoff 0
	$menubar add cascade -label Help -menu $menubar.help

	$menubar.help add command -label "About IMCA GUI" \
			-command menubar_about_command

	$menubar.help add command -label "Help for SER-CAT 22-ID" \
			-command [ itcl::code display_help "ser22id.html" ]

	proc menubar_about_command {} {
		set t "IMCA GUI was written by William Lavender based on "
		append t "a design by the IMCA-CAT staff."

		tk_dialog .about_dialog "About IMCA GUI" $t info 0 "Done"
	}
}

